@@ -1,21 +1,21 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1998-2000 Thomas Roessler <roessler@guug.de>
- * 
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1998-2001,2007 Thomas Roessler <roessler@does-not-exist.org>
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
- */ 
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 
 /*
  * Completely stolen from the Mutt mailreader
@@ -36,7 +36,7 @@
 #include <limits.h>
 
 #ifndef _POSIX_PATH_MAX
-#include <posix1_lim.h>
+#include <limits.h>
 #endif
 
 #include "dotlock.h"
@@ -56,7 +56,7 @@ static int Retry = MAXLOCKATTEMPT;
 
 static char *Hostname;
 
-static int dotlock_deference_symlink (char *, size_t, const char *);
+static int dotlock_dereference_symlink (char *, size_t, const char *);
 static int dotlock_prepare (char *, size_t, const char *, int fd);
 static int dotlock_check_stats (struct stat *, struct stat *);
 static int dotlock_dispatch (const char *, int fd);
@@ -85,7 +85,7 @@ int main (int argc, char **argv)
   struct utsname utsname;
 
   /* determine the system's host name */
-  
+
   uname (&utsname);
   if (!(Hostname = safe_strdup (utsname.nodename)))
     return DL_EX_ERROR;
@@ -95,7 +95,7 @@ int main (int argc, char **argv)
 
   /* parse the command line options. */
   DotlockFlags = 0;
-  
+
   while ((i = getopt (argc, argv, "dtfur:")) != EOF)
   {
     switch (i)
@@ -108,7 +108,7 @@ int main (int argc, char **argv)
       /* other flags */
       case 'f': DotlockFlags |= DL_FL_FORCE; break;
       case 'r': DotlockFlags |= DL_FL_RETRY; Retry = atoi (optarg); break;
-      
+
       default: usage (argv[0]);
     }
   }
@@ -130,7 +130,7 @@ static int dotlock_dispatch (const char *f, int fd)
    * `dirname $f`.  Additionally, f has been opened for
    * reading to verify that the user has at least read
    * permissions on that file.
-   * 
+   *
    * For a more detailed explanation of all this, see the
    * lengthy comment below.
    */
@@ -153,12 +153,12 @@ static int dotlock_dispatch (const char *f, int fd)
 
 /*
  * Usage information.
- * 
+ *
  * This function doesn't return.
- * 
+ *
  */
 
-static void 
+static void
 usage (const char *av0)
 {
   fprintf (stderr, "usage: %s [-t|-f|-u|-d] [-r <retries>] file\n", av0);
@@ -170,7 +170,7 @@ usage (const char *av0)
 	"\n  -d\t\tunlink"
 	"\n  -r <retries>\tRetry locking"
 	"\n", stderr);
-  
+
   exit (DL_EX_ERROR);
 }
 
@@ -178,48 +178,48 @@ usage (const char *av0)
 /*
  * Access checking: Let's avoid to lock other users' mail
  * spool files if we aren't permitted to read them.
- * 
+ *
  * Some simple-minded access (2) checking isn't sufficient
  * here: The problem is that the user may give us a
  * deeply nested path to a file which has the same name
  * as the file he wants to lock, but different
  * permissions, say, e.g.
  * /tmp/lots/of/subdirs/var/spool/mail/root.
- * 
+ *
  * He may then try to replace /tmp/lots/of/subdirs by a
  * symbolic link to / after we have invoked access () to
  * check the file's permissions.  The lockfile we'd
  * create or remove would then actually be
  * /var/spool/mail/root.
- * 
+ *
  * To avoid this attack, we proceed as follows:
- * 
+ *
  * - First, follow symbolic links a la
- *   dotlock_deference_symlink ().
- * 
+ *   dotlock_dereference_symlink ().
+ *
  * - get the result's dirname.
- * 
+ *
  * - chdir to this directory.  If you can't, bail out.
- * 
+ *
  * - try to open the file in question, only using its
  *   basename.  If you can't, bail out.
- * 
+ *
  * - fstat that file and compare the result to a
  *   subsequent lstat (only using the basename).  If
  *   the comparison fails, bail out.
- * 
+ *
  * dotlock_prepare () is invoked from main () directly
  * after the command line parsing has been done.
  *
  * Return values:
- * 
+ *
  * 0 - Evereything's fine.  The program's new current
  *     directory is the contains the file to be locked.
  *     The string pointed to by bn contains the name of
  *     the file to be locked.
- * 
+ *
  * -1 - Something failed. Don't continue.
- * 
+ *
  * tlr, Jul 15 1998
  */
 
@@ -233,20 +233,20 @@ dotlock_check_stats (struct stat *fsb, struct stat *lsb)
 
   if (S_ISLNK (lsb->st_mode) || S_ISLNK (fsb->st_mode))
     return -1;
-  
+
   if ((lsb->st_dev != fsb->st_dev) ||
-     (lsb->st_ino != fsb->st_ino) ||
-     (lsb->st_mode != fsb->st_mode) ||
-     (lsb->st_nlink != fsb->st_nlink) ||
-     (lsb->st_uid != fsb->st_uid) ||
-     (lsb->st_gid != fsb->st_gid) ||
-     (lsb->st_rdev != fsb->st_rdev) ||
-     (lsb->st_size != fsb->st_size))
+      (lsb->st_ino != fsb->st_ino) ||
+      (lsb->st_mode != fsb->st_mode) ||
+      (lsb->st_nlink != fsb->st_nlink) ||
+      (lsb->st_uid != fsb->st_uid) ||
+      (lsb->st_gid != fsb->st_gid) ||
+      (lsb->st_rdev != fsb->st_rdev) ||
+      (lsb->st_size != fsb->st_size))
   {
     /* something's fishy */
     return -1;
   }
-  
+
   return 0;
 }
 
@@ -259,10 +259,10 @@ dotlock_prepare (char *bn, size_t l, const char *f, int _fd)
   char *p;
   int fd;
   int r;
-  
-  if (dotlock_deference_symlink (realpath, sizeof (realpath), f) == -1)
+
+  if (dotlock_dereference_symlink (realpath, sizeof (realpath), f) == -1)
     return -1;
-  
+
   if ((p = strrchr (realpath, '/')))
   {
     *p = '\0';
@@ -277,9 +277,9 @@ dotlock_prepare (char *bn, size_t l, const char *f, int _fd)
 
   if (strlen (basename) + 1 > l)
     return -1;
-  
+
   strfcpy (bn, basename, l);
-  
+
   if (chdir (dirname) == -1)
     return -1;
 
@@ -287,15 +287,15 @@ dotlock_prepare (char *bn, size_t l, const char *f, int _fd)
     fd = _fd;
   else if ((fd = open (basename, O_RDONLY)) == -1)
     return -1;
-  
+
   r = fstat (fd, &fsb);
-  
+
   if (_fd == -1)
     close (fd);
-  
+
   if (r == -1)
     return -1;
-  
+
   if (lstat (basename, &lsb) == -1)
     return -1;
 
@@ -307,13 +307,13 @@ dotlock_prepare (char *bn, size_t l, const char *f, int _fd)
 
 /*
  * Expand a symbolic link.
- * 
+ *
  * This function expects newpath to have space for
  * at least _POSIX_PATH_MAX characters.
  *
  */
 
-static void 
+static void
 dotlock_expand_link (char *newpath, const char *path, const char *link)
 {
   const char *lb = NULL;
@@ -340,20 +340,20 @@ dotlock_expand_link (char *newpath, const char *path, const char *link)
 
 
 /*
- * Deference a chain of symbolic links
- * 
+ * Dereference a chain of symbolic links
+ *
  * The final path is written to d.
  *
  */
 
 static int
-dotlock_deference_symlink (char *d, size_t l, const char *path)
+dotlock_dereference_symlink (char *d, size_t l, const char *path)
 {
   struct stat sb;
   char realpath[_POSIX_PATH_MAX];
   const char *pathptr = path;
   int count = 0;
-  
+
   while (count++ < MAXLINKS)
   {
     if (lstat (pathptr, &sb) == -1)
@@ -361,19 +361,19 @@ dotlock_deference_symlink (char *d, size_t l, const char *path)
       /* perror (pathptr); */
       return -1;
     }
-    
+
     if (S_ISLNK (sb.st_mode))
     {
       char linkfile[_POSIX_PATH_MAX];
       char linkpath[_POSIX_PATH_MAX];
       int len;
 
-      if ((len = readlink (pathptr, linkfile, sizeof (linkfile))) == -1)
+      if ((len = readlink (pathptr, linkfile, sizeof (linkfile) - 1)) == -1)
       {
 	/* perror (pathptr); */
 	return -1;
       }
-      
+
       linkfile[len] = '\0';
       dotlock_expand_link (linkpath, pathptr, linkfile);
       strfcpy (realpath, linkpath, sizeof (realpath));
@@ -389,11 +389,11 @@ dotlock_deference_symlink (char *d, size_t l, const char *path)
 
 /*
  * Dotlock a file.
- * 
+ *
  * realpath is assumed _not_ to be an absolute path to
  * the file we are about to lock.  Invoke
  * dotlock_prepare () before using this function!
- * 
+ *
  */
 
 #define HARDMAXATTEMPTS 10
@@ -409,11 +409,12 @@ dotlock_lock (const char *realpath)
   int hard_count = 0;
   struct stat sb;
   time_t t;
-  
-  sprintf (nfslockfile, "%s.%s.%d", realpath, Hostname, (int) getpid ());
-  sprintf (lockfile, "%s.lock", realpath);
 
-  
+  snprintf (nfslockfile, sizeof (nfslockfile), "%s.%s.%d",
+            realpath, Hostname, (int) getpid ());
+  snprintf (lockfile, sizeof (lockfile), "%s.lock", realpath);
+
+
   unlink (nfslockfile);
 
   while ((fd = open (nfslockfile, O_WRONLY | O_EXCL | O_CREAT, 0)) < 0)
@@ -426,9 +427,9 @@ dotlock_lock (const char *realpath)
   }
 
 
-  
+
   close (fd);
-  
+
   while (hard_count++ < HARDMAXATTEMPTS)
   {
     link (nfslockfile, lockfile);
@@ -459,19 +460,20 @@ dotlock_lock (const char *realpath)
 	return DL_EX_EXIST;
       }
     }
-    
+
     prev_size = sb.st_size;
-    
+
     /* don't trust sleep (3) as it may be interrupted
-     * by users sending signals. 
+     * by users sending signals.
      */
-    
+
     t = time (NULL);
-    do {
+    do
+    {
       sleep (1);
     } while (time (NULL) == t);
   }
-  
+
   unlink (nfslockfile);
 
   return DL_EX_OK;
@@ -479,10 +481,10 @@ dotlock_lock (const char *realpath)
 
 
 /*
- * Unlock a file. 
- * 
+ * Unlock a file.
+ *
  * The same comment as for dotlock_lock () applies here.
- * 
+ *
  */
 
 static int
@@ -491,13 +493,14 @@ dotlock_unlock (const char *realpath)
   char lockfile[_POSIX_PATH_MAX + LONG_STRING];
   int i;
 
-  sprintf (lockfile, "%s.lock", realpath);
-  
+  snprintf (lockfile, sizeof (lockfile), "%s.lock",
+            realpath);
+
   i = unlink (lockfile);
-  
+
   if (i == -1)
     return DL_EX_ERROR;
-  
+
   return DL_EX_OK;
 }
 
@@ -523,9 +526,9 @@ dotlock_unlink (const char *realpath)
 
 /*
  * Check if a file can be locked at all.
- * 
+ *
  * The same comment as for dotlock_lock () applies here.
- * 
+ *
  */
 
 static int
