@@ -1,26 +1,25 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
- * 
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
- *     along with this program; if not, write to the Free Software Foundation,
- *     Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,, USA.
- */ 
+ *     along with this program; if not, write to the Free Software
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 
-/* $Id: rfc2047.h,v 1.3 2005-10-29 14:48:11 roland Exp $ */
+#include <sys/types.h>
+#include "rfc822.h"
 
-void rfc2047_encode_string (char *, size_t, const unsigned char *);
-void rfc2047_encode_adrlist (ADDRESS *);
+extern const char *Charset;
 
 void rfc2047_decode (char *, const char *, size_t);
 void rfc2047_decode_adrlist (ADDRESS *);
-
